@@ -18,43 +18,49 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h" //For GLOM_ENABLE_POSTGRESQL
 #include "dialog_connection.h"
-#include "box_db.h" //For Box_DB::connect_to_server().
 #include <glibmm/i18n.h>
 
+#include <libglom/connectionpool.h>
+
+#ifdef GLOM_ENABLE_POSTGRESQL
+#include <libglom/connectionpool_backends/postgres_central.h>
+#include <libglom/connectionpool_backends/postgres_self.h>
+#endif //#ifdef GLOM_ENABLE_POSTGRESQL
+
 namespace Glom
 {
 
-Dialog_Connection::Dialog_Connection(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
+Dialog_Connection::Dialog_Connection(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Gtk::Dialog(cobject),
   Base_DB(),
   m_entry_host(0),
   m_entry_user(0),
   m_entry_password(0),
-  m_label_database(0)
+  m_label_database(0),
+  m_label_note(0)
 {
-  refGlade->get_widget("entry_host", m_entry_host);
-  refGlade->get_widget("entry_user", m_entry_user);
-  refGlade->get_widget("entry_password", m_entry_password);
-  refGlade->get_widget("label_database", m_label_database);
+  builder->get_widget("entry_host", m_entry_host);
+  builder->get_widget("entry_user", m_entry_user);
+  builder->get_widget("entry_password", m_entry_password);
+  builder->get_widget("label_database", m_label_database);
+  builder->get_widget("connection_note", m_label_note);
 
 #ifdef GLOM_ENABLE_MAEMO
   // Make the bold title the window title (which cannot be empty in maemo
   // because it displays <Untitled window> instead). This also helps to
   // make the dialog smaller in height, so we save a bit screen space required
   // by the onscreen keyboard.
-  Gtk::Label* title;
-  Gtk::Label* note;
-
-  refGlade->get_widget("connection_title", title);
-  refGlade->get_widget("connection_note", note);
+  Gtk::Label* title = 0;
+  builder->get_widget("connection_title", title);
 
   set_title(title->get_text());
   title->hide();
 
   // Without size request, this label enlarges the dialog significantly,
   // and the text is still truncated.
-  note->set_size_request(400, -1);
+  m_label_note->set_size_request(400, -1);
 #endif
 }
 
@@ -70,7 +76,7 @@ sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection
 {
   //std::cout << "debug: Dialog_Connection::connect_to_server_with_connection_settings()" << std::endl;
 
-  //TODO: Bakery::BusyCursor busy_cursor(get_app_window());
+  //TODO: BusyCursor busy_cursor(get_app_window());
 
   sharedptr<SharedConnection> result(0);
 
@@ -80,42 +86,72 @@ sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection
     //Set the connection details in the ConnectionPool singleton.
     //The ConnectionPool will now use these every time it tries to connect.
 
-    const Document_Glom* document = get_document();
+    const Document* document = get_document();
     if(document)
     {
       //std::cout << "debug: Dialog_Connection::connect_to_server_with_connection_settings(): m_database_name=" << m_database_name << std::endl;
       connection_pool->set_database(m_database_name);
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-      if(document->get_connection_is_self_hosted())
-        connection_pool->set_host("localhost");
-      else
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-        connection_pool->set_host(m_entry_host->get_text());
- 
+#ifdef GLOM_ENABLE_POSTGRESQL
+      if(document->get_hosting_mode() == Document::HOSTING_MODE_POSTGRES_CENTRAL)
+      {
+        ConnectionPool::Backend* backend = connection_pool->get_backend();
+        ConnectionPoolBackends::PostgresCentralHosted* central = dynamic_cast<ConnectionPoolBackends::PostgresCentralHosted*>(backend);
+        g_assert(central);
+
+        central->set_host(m_entry_host->get_text());
+      }
+#endif //GLOM_ENABLE_POSTGRESQL
+
       connection_pool->set_user(m_entry_user->get_text());
       connection_pool->set_password(m_entry_password->get_text());
+
       //if(document)
       //{
       //  connection_pool->set_database(document->get_connection_database());
       //}
     }
 
-    connection_pool->set_ready_to_connect(); //Box_DB::connect_to_server() will now attempt the connection-> Shared instances of m_Connection will also be usable.
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-    result = Box_DB::connect_to_server(const_cast<Dialog_Connection*>(this));
+    result = Base_DB::connect_to_server(const_cast<Dialog_Connection*>(this));
 #else
-    result = Box_DB::connect_to_server(const_cast<Dialog_Connection*>(this), error);
+    result = Base_DB::connect_to_server(const_cast<Dialog_Connection*>(this), error);
 #endif
 
-    /*
+#ifdef GLOM_ENABLE_POSTGRESQL
     if(document)
     {
-      document->set_connection_server(m_entry_host->get_text());
-      document->set_connection_user(m_entry_user->get_text());
+      //Remember the port, 
+      //to make opening faster next time,
+      //and so we can tell connecting clients (using browse network) what port to use:
+      Document* unconst = const_cast<Document*>(document);
+
+      if(document->get_hosting_mode() == Document::HOSTING_MODE_POSTGRES_CENTRAL)
+      {
+        ConnectionPool::Backend* backend = connection_pool->get_backend();
+        ConnectionPoolBackends::PostgresCentralHosted* central = dynamic_cast<ConnectionPoolBackends::PostgresCentralHosted*>(backend);
+        g_assert(central != NULL);
+
+        unconst->set_connection_port(central->get_port() );
+	// As we know the port of the database already, we don't need to try
+	// other ports anymore:
+        unconst->set_connection_try_other_ports(false);
+      }
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+      else if(document->get_hosting_mode() == Document::HOSTING_MODE_POSTGRES_SELF)
+      {
+        ConnectionPool::Backend* backend = connection_pool->get_backend();
+        ConnectionPoolBackends::PostgresSelfHosted* self = dynamic_cast<ConnectionPoolBackends::PostgresSelfHosted*>(backend);
+        g_assert(self != NULL);
+
+        unconst->set_connection_port(self->get_port() );
+        unconst->set_connection_try_other_ports(false);
+      }
+#endif //GLOM_ENABLE_CLIENT_ONLY
     }
-    */
+#endif //GLOM_ENABLE_POSTGRESQL
   }
   else
      std::cerr << "Dialog_Connection::connect_to_server_with_connection_settings(): ConnectionPool::get_instance() failed." << std::endl;
@@ -125,17 +161,19 @@ sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection
 
 void Dialog_Connection::load_from_document()
 {
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(document)
   {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+#ifdef GLOM_ENABLE_POSTGRESQL
     //Load server and user:
-    if(document->get_connection_is_self_hosted())
+    if(document->get_hosting_mode() != Document::HOSTING_MODE_POSTGRES_CENTRAL)
     {
        m_entry_host->set_text("(self hosted)");
        m_entry_host->set_sensitive(false);
     }
     else
+#endif // LOM_ENABLE_POSTGRESQL
 #endif // !GLOM_ENABLE_CLIENT_ONLY
     {
       Glib::ustring host = document->get_connection_server();
@@ -167,9 +205,25 @@ void Dialog_Connection::load_from_document()
 
 }
 
+///Disable irrelevant fields:
+void Dialog_Connection::set_connect_to_browsed()
+{
+  m_entry_host->set_sensitive(false);
+}
+
 void Dialog_Connection::set_self_hosted_user_and_password(const Glib::ustring& user, const Glib::ustring& password)
+{
+  set_username(user);
+  set_password(password);
+}
+
+void Dialog_Connection::set_username(const Glib::ustring& user)
 {
   m_entry_user->set_text(user);
+}
+
+void Dialog_Connection::set_password(const Glib::ustring& password)
+{
   m_entry_password->set_text(password);
 }
 
@@ -182,4 +236,15 @@ void Dialog_Connection::set_database_name(const Glib::ustring& name)
     m_label_database->set_text(name);
 }
 
+void Dialog_Connection::get_username_and_password(Glib::ustring& username, Glib::ustring& password) const
+{
+  username = m_entry_user->get_text();
+  password = m_entry_password->get_text();
+}
+
+void Dialog_Connection::set_confirm_existing_user_note()
+{
+  m_label_note->set_text(_(""));
+}
+
 } //namespace Glom
